# CSS Box Sizing Module Level 3

## Editor’s Draft, 1 January 1970

Specification Metadata

This version:
  https://drafts.csswg.org/css-sizing-3/
Latest published version:
  https://www.w3.org/TR/css-sizing-3/
Previous Versions:
  https://www.w3.org/TR/2019/WD-css-sizing-3-20190522/
  https://www.w3.org/TR/2018/WD-css-sizing-3-20180304/
  https://www.w3.org/TR/2017/WD-css-sizing-3-20170207/
Editors:
  Tab Atkins (Google)
  Elika J. Etemad / fantasai (Invited Expert)
Issue Tracking:
  GitHub Issues
Suggest an Edit for this Spec:
  GitHub Editor

Copyright © 1970 W3C® (MIT, ERCIM, Keio, Beihang). W3C liability, trademark and permissive document license rules apply.

## Abstract

This module extends the CSS sizing properties with keywords that represent content-based "intrinsic" sizes and context-based "extrinsic" sizes, allowing CSS to more easily describe boxes that fit their content or fit into a particular layout context.

CSS is a language for describing the rendering of structured documents (such as HTML and XML) on screen, on paper, etc.

## Status of this document

This is a public copy of the editors’ draft. It is provided for discussion only and may change at any moment. Its publication here does not imply endorsement of its contents by W3C. Don’t cite this document other than as work in progress.

GitHub Issues are preferred for discussion of this specification. When filing an issue, please put the text “css-sizing” in the title, preferably like this: “[css-sizing] …summary of comment…”. All issues and comments are archived, and there is also a historical archive.

This document was produced by the CSS Working Group.

This document was produced by a group operating under the W3C Patent Policy. W3C maintains a public list of any patent disclosures made in connection with the deliverables of the group; that page also includes instructions for disclosing a patent. An individual who has actual knowledge of a patent which the individual believes contains Essential Claim(s) must disclose the information in accordance with section 6 of the W3C Patent Policy.

This document is governed by the 1 March 2019 W3C Process Document.

## 1. Introduction

This section is not normative.

CSS layout has several different concepts of automatic sizing that are used in various layout calculations. This section defines some more precise terminology to help connect the layout behaviors of this spec to the calculations used in other modules, and some new keywords for the width and height properties to allow authors to assign elements the dimensions resulting from these size calculations.

This spec needs illustrations! See issue.

### 1.1. Module interactions

This module extends the width, height, min-width, min-height, max-width, max-height, and column-width features defined in [CSS2] chapter 10 and in [CSS3COL]

The definition of the box-sizing property in this module supersedes the one in [CSS-UI-3].

### 1.2. Values

This specification follows the CSS property definition conventions from [CSS2]. Value types not defined in this specification are defined in CSS Values & Units [CSS-VALUES-3]. Other CSS modules may expand the definitions of these value types.

In addition to the property-specific values listed in their definitions, all properties defined in this specification also accept the CSS-wide keywords keywords as their property value. For readability they have not been repeated explicitly.

## 2. Terminology

size
  A one- or two-dimensional measurement: a block size and/or inline size; alternatively a width and/or height.
  Whether the width or height corresponds to an inline size or block size depends on the writing mode.
inner size
  The content-box size of a box.
outer size
  The margin-box size of a box.
definite size
  A size that can be determined without performing layout; that is, a <length>, a measure of text (without consideration of line-wrapping), a size of the initial containing block, or a <percentage> or other formula (such the “stretch-fit” sizing of non-replaced blocks [CSS2]) that is resolved solely against definite sizes.

  Additionally, the size of the containing block of an absolutely positioned element is always definite with respect to that element.

indefinite size
  A size that is not definite. Indefinite available space is essentially infinite.
available space
  A size representing the space into which a box is laid out, as determined by the rules of the formatting context in which it participates. The space available to a box is usually either a measurement of its containing block (if that is definite) or an infinite size (when it is indefinite). Available space can alternatively be either a min-content constraint or a max-content constraint.
stretch fit
  The stretch fit into a given size is that size, minus the element’s computed margins (not collapsed, treating auto as zero), border, and padding in the given dimension (such that the outer size is a perfect fit), and flooring at zero (so that the inner size is not negative).

  Note: This is the formula used to calculate the auto widths of non-replaced blocks in normal flow in CSS2.1§10.3.3.

fallback size
  Some sizing algorithms do not work well with an infinite size. In these cases, the fallback size is used instead. Unless otherwise specified, this is the size of the initial containing block.

### 2.1. Auto Box Sizes

There are four types of automatically-determined sizes in CSS (sizes resulting from auto sizing rules, depending on context):

stretch-fit size
stretch-fit inline size
stretch-fit block size
  The size a box would take if its outer size filled the available space in the given axis; in other words, the stretch fit into the available space, if that is definite. Undefined if the available space is indefinite.

  Note: For the inline axis, this is called the “available width” in CSS2.1§10.3.5 and computed by the rules in CSS2.1§10.3.3.

  Note: Calculations involving this size need to specify a fallback behavior for when the available space is indefinite if that happens to be possible.

max-content size
  A box’s “ideal” size in a given axis when given infinite available space. Usually this is the smallest size the box could take in that axis while still fitting around its contents, i.e. minimizing unfilled space while avoiding overflow.
  max-content inline size

    The box’s “ideal” size in the inline axis. Usually the narrowest inline size it could take while fitting around its contents if none of the soft wrap opportunities within the box were taken. (See § 4 Intrinsic Size Determination.)

    Note: This is called the “preferred width” in CSS2.1§10.3.5 and the “maximum cell width” in CSS2.1§17.5.2.2.

  max-content block size

    The box’s “ideal” size in the block axis. Usually the block size of the content after layout.

min-content size
  The smallest size a box could take that doesn’t lead to overflow that could be avoided by choosing a larger size. (See § 4 Intrinsic Size Determination.)
  min-content inline size

    The narrowest inline size a box could take that doesn’t lead to inline-dimension overflow that could be avoided by choosing a larger inline size. Roughly, the inline size that would fit around its contents if all soft wrap opportunities within the box were taken.

    Note: This is called the “preferred minimum width” in CSS2.1§10.3.5 and the “minimum content width” in CSS2.1§17.5.2.2.

  min-content block size

    In general, and definitely for block containers, tables, and inline boxes, this is equivalent to the max-content block size.

fit-content size
fit-content inline size
fit-content block size
  If the available space in a given axis is definite, equal to `min(max-content size, max(min-content size, stretch-fit size))`. Otherwise, equal to the max-content size in that axis.

  Note: This is called the “shrink-to-fit” width in CSS2.1§10.3.5 and CSS Multi-column Layout § 3.4.

intrinsic size
  A max-content size or min-content size, i.e. a size arising primarily from the size of the content. (Some uses of this term may refer also to sizes derived primarily from one of these two sizes.)

### 2.2. Intrinsic Size Contributions

max-content contribution
  The size that a box contributes to its containing block’s max-content size.
min-content contribution
  The size that a box contributes to its containing block’s min-content size.
intrinsic size contribution
  A max-content contribution, min-content contribution, or similarly-calculated content-based size contribution.

Intrinsic size contributions are based on the outer size of the box; for this purpose auto margins are treated as zero.

### 2.3. Intrinsic Size Constraints

max-content constraint
  A sizing constraint imposed by the box’s containing block that causes it to produce its max-content contribution.
min-content constraint
  A sizing constraint imposed by the box’s containing block that causes it to produce its min-content contribution.

## 3. Specifying Box Sizes

### 3.1. Sizing Properties

This section defines the sizing properties width, height, min-width, min-height, max-width, and max-height. Their potential values are defined in the next section, § 3.2 Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values.

Note: Additional flow-relative aliases to these properties are defined in [CSS-LOGICAL-1].

We would like to define shorthands for each pair of sizing properties (e.g. width and height) but there is a naming conflict with the @page size descriptor [CSS-PAGE-3], so this has been deferred to Level 4. Suggestions on how to resolve this problem are welcome, see discussion.

#### 3.1.1. Preferred Size Properties: the width and height properties

Name: | width, height
Value: | auto | <length-percentage> | min-content | max-content | fit-content(<length-percentage>)
Initial: | auto
Applies to: | all elements except non-replaced inlines
Inherited: | no
Percentages: | relative to width/height of containing block
Computed value: | as specified, with <length-percentage> values computed
Canonical order: | per grammar
Animation type: | by computed value type, recursing into fit-content()

The width and height properties specify the preferred (physical) width and height of the box, respectively.

#### 3.1.2. Minimum Size Properties: the min-width and min-height properties

Name: | min-width, min-height
Value: | auto | <length-percentage> | min-content | max-content | fit-content(<length-percentage>)
Initial: | auto
Applies to: | all elements that accept width or height
Inherited: | no
Percentages: | relative to width/height of containing block
Computed value: | as specified, with <length-percentage> values computed
Canonical order: | per grammar
Animatable: | by computed value, recursing into fit-content()

The min-width and min-height properties specify the minimum width (or “min width”) and minimum height (or “min height”) of the box, respectively.

Note: The initial value of auto is new; in [CSS2] the initial value was zero.

#### 3.1.3. Maximum Size Properties: the max-width and max-height properties

Name: | max-width, max-height
Value: | none | <length-percentage> | min-content | max-content | fit-content(<length-percentage>)
Initial: | none
Applies to: | all elements that accept width or height
Inherited: | no
Percentages: | relative to width/height of containing block
Computed value: | as specified, with <length-percentage> values computed
Canonical order: | per grammar
Animatable: | by computed value, recursing into fit-content()

The max-width and max-height properties specify the maximum width (or “max width”) and maximum height (or “max height”) of the box, respectively.

### 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values

<length-percentage>
  Specifies the size of the box using <length> and/or <percentage>. The box-sizing property indicates whether the content box or border box is measured.

  Percentages are resolved against the width/height, as appropriate, of the box’s containing block. If, in a particular axis, the containing block’s size depends on the box’s size, see the relevant layout module for special rules on how to resolve percentages.

  Negative values are invalid.

auto
  For width/height, specifies an automatic size. See the relevant layout module for how to calculate this.

  For min-width/min-height, specifies an automatic minimum size. Unless otherwise defined by the relevant layout module, however, it resolves to a used value of 0. For backwards-compatibility, the resolved value of this keyword is zero for boxes of all [CSS2] display types: block and inline boxes, inline blocks, and all the table layout boxes. It also resolves to zero when no box is generated.

none
  No limit on the size of the box.
min-content
  Use the min-content size in the relevant axis; for a box’s block size, unless otherwise specified, this is equivalent to its automatic size.
max-content
  Use the min-content size in the relevant axis; for a box’s block size, unless otherwise specified, this is equivalent to its automatic size.
fit-content(<length-percentage>)
  Use the fit-content formula with the available space replaced by the specified argument, i.e. `min(max-content, max(min-content, <length-percentage>))`.

  For intrinsic size contributions, ''fit-content(<length>)'' behaves as the resulting length; fit-content() containing a <percentage> behaves as min-content for min-content contributions, and as max-content for max-content contributions.

  Negative values are invalid.

In all cases, the used value is floored to preserve a non-negative inner size.

Note: The min-content, max-content, and fit-content() values are new in Level 3.

Note: The flex-basis property hereby also gains these new keywords, as its values are defined by reference to <width>.

Note: This section previously defined stretch and fit-content as keywords representing the stretch-fit size and fit-content size, respectively. These keywords have been deferred to Level 4 (along with an additional contain keyword that behaves similarly to stretch but preserves the intrinsic aspect ratio, if any) to better work out the implications in situations with indefinite available space.

#### 3.2.1. “Behaving as auto”

To have a common term for both when width/height computes to auto and when it is defined to behave as if auto were specified (as in the case of block percentage heights resolving against an indefinite size, see CSS2§10.5), the property is said to behave as auto in both of these cases.

Note: Legacy spec prose defining layout behavior might explicitly refer to width/height having a computed value of auto as a condition; these should be interpreted as meaning behaves as auto, and reported to the CSSWG for updating.

Replace this section with references to the new term automatic size.

#### 3.2.2. Containing or Excluding Floats

This section is non-normative.

Although block box boundaries are typically pervious to floats, sometimes an author needs them to contain their own (descendant) floats or to exclude floats from outside. For Block layout, specifying display: flow-root will make the box a formatting context root, which has this behavior.

Note: Boxes participating in Flex, Grid, or Table layout will automatically have this behavior.

### 3.3. Box Edges for Sizing: the box-sizing property

Name: | box-sizing
Value: | content-box | border-box
Initial: | content-box
Applies to: | all elements that accept width or height
Inherited: | no
Percentages: | N/A
Computed value: | specified keyword
Canonical order: | per grammar
Animation type: | discrete

The box-sizing property defines whether fixed sizes (such as <length>s and <percentage>s) are assigned to the content box or to the border box. It affects the interpretation of all sizing properties, including flex-basis.

Values have the following meanings:

content-box
  Sizes specified on sizing properties as <length-percentage> represent the box’s inner sizes, excluding the margins/border/padding: they are applied to the content box. The padding and border of the box are laid out and drawn outside the specified width and height.

  Note: This is the behavior of width and height as specified by CSS2.1, and is thus the default.

border-box
  Sizes specified on sizing properties as <length-percentage> represent the box’s visually-apparent sizes, including the borders/padding (but not margin): they are applied to the border box. The padding and border of the box are laid out and drawn inside the specified width and height, with the content box sized to fill the remaining space, floored at zero.

  The content width and height are calculated by subtracting the border and padding widths of the respective sides from the specified <length-percentage>. As the content width and height cannot be negative, this computation is floored at zero.

  Used values, as exposed for instance through `getComputedStyle()`, also refer to the border box.

Values affected by box-sizing include both raw <length-percentage> values and those used in functional notations such as fit-content(). In contrast, non-quantitative values such as auto and min-content are not influenced by the box-sizing property (unless otherwise specified).

For example, the following properties set the content-box size of the box to 100px, with the border-box size calculating to 120px:

    .box {
      box-sizing:   content-box; /* default */
      width:        100px;
      padding-left: 10px;
      border-left:  10px solid;
    }

On the other hand, by changing to border-box, the border-box is set to 100px, with the content-box size calculating to 80px:

    .box {
      box-sizing:   border-box;
      width:        100px;
      padding-left: 10px;
      border-left:  10px solid;
    }

The inner size can’t be less than zero, so if the padding + border is greater than the specified border-box size, the box will end up larger than specified. In this case, the content-box size will floor at 0px so the border-box size ends up at 120px, even though width: 100px is specified for the border box:

    .box {
      box-sizing:   border-box;
      width:        100px;
      padding-left: 60px;
      border-left:  60px solid;
      /* padding + border = 120px */
    }

This example uses box-sizing to evenly horizontally split two divs with fixed size borders inside a div container, which would otherwise require additional markup.

sample CSS:

    div.container {
      width:38em;
      border:1em solid black;
    }

    div.split {
      box-sizing:border-box;
      width:50%;
      border:1em silver ridge;
      float:left;
    }

sample HTML fragment:

    <div class="container">
    <div class="split">This div occupies the left half.</div>
    <div class="split">This div occupies the right half.</div>
    </div>

demonstration of sample CSS and HTML:

This div should occupy the left half.
This div should occupy the right half.
The two divs above should appear side by side, each (including borders) 50% of the content width of their container. If instead they are stacked one on top of the other then your browser does not support box-sizing.

Note: Certain HTML elements, such as `button`, default to border-box behavior. See HTML for details on which elements have this behavior.

In legacy CSS specifications, the terms width, height, minimum (min) width, minimum (min) height, maximum (max) width, and maximum (max) height generally refer to the inner size (content-box size) of a box unless otherwise indicated.

Refer to CSS User Interface 3 §3.1 Changing the Box Model: the box-sizing property for an explicit disambiguation of these terms for the Visual formatting model details section of [CSS2].

To avoid ambiguities, specification authors should avoid ambiguous uses of terms such as width or height without further qualification, and should explicitely refer and link to the inner size, the outer size, the size of the border-box, the computed value of the sizing properties, etc, as appropriate for each case.

### 3.4. New Column Sizing Values: the min-content, max-content, and fit-content() values

Name: | column-width
New values: | min-content | max-content | fit-content(<length-percentage>)
Computed value: | as specified, with <length-percentage> values computed
Animation type: | by computed value type

When used as values for column-width, the new keywords specify the optimal column width:

min-content
  Specifies the optimal column width as the min-content inline size of the multi-column container’s contents.
max-content
  Specifies the optimal column width as the max-content inline size of the multi-column container’s contents.
fit-content(<length-percentage>)
  Specifies the optimal column width as `min(max-content size, max(min-content size, <length-percentage>))`

Note: The column width never varies by column. When the column width is informed by the multi-column container’s contents (as in the keywords above), all of its contents are taken under consideration and the calculated width is shared by all the columns.

## 4. Intrinsic Size Determination

Intrinsic sizing determines sizes based on the contents of an element, without regard for its context.

### 4.1. Intrinsic Sizes

The min-content size of a box in each axis is the size it would have if it was a float given an auto size in that axis (and no minimum or maximum size in that axis) and if its containing block was zero-sized in that axis. (In other words, the minimum size it has when sized as “shrink-to-fit”.)

The max-content size of a box in each axis is the size it would have if it was a float given an auto size in that axis (and no minimum or maximum size in that axis), and if its containing block was infinitely-sized in that axis. (In other words, the maximum size it has when sized as “shrink-to-fit”.)

The min-content size and max-content size are collectively referred to as the intrinsic sizes.

Note: Note that when the box has an intrinsic aspect ratio, size constraints in the opposite dimension will transfer through and can affect the auto size in the considered one. See CSS2§10.

For replaced elements, if the max-content size size would depend on the size of the containing block (as it does for images with an intrinsic aspect ratio but no intrinsic size, see CSS2§10.3.2) and therefore can’t be calculated against an infinitely-sized containing block then:

For boxes with an intrinsic aspect ratio, but no intrinsic size:

  - If the available space is definite in the inline axis, use the stretch fit into that size for the inline size and calculate the block size using the aspect ratio.

  - Otherwise if the box has a definite value for min-width or min-height, use that size and calculate the other dimension using the aspect ratio; if both dimensions have a definite minimum, choose the one that results in the larger overall size.

    Note: This case was previous calculated from a 300x150 default size, rather than the box’s min size. This is believed to be a better behavior, and likely to be web compatible, but please send feedback to the CSSWG if there are any problems.

  - Otherwise use a width of 300px (height of 150px in vertical writing modes) and calculate the other dimension using the aspect ratio.

For boxes without an intrinsic aspect ratio:

  - If the available space is definite in the appropriate dimension, use the stretch fit into that size in that dimension.

  - Otherwise, if the box has a definite non-zero minimum size (min-width/min-height) in that dimension, use that size.

  - Otherwise, use 300px for the width and/or 150px for the height as needed.

Since a block-level or inline-level replaced element whose height or width behaves as auto is effectively defined to use its max-content size (CSS2§10.3.2), this specification applies the rules above to the undefined case of a replaced element whose height and width both behave as auto.

Note: This specification does not define how to determine the size of a float. Please refer to [CSS2], the relevant CSS specification for that display type, and/or existing implementations for further details. A future specification will define this in detail, replacing the CSS2 “definition”, such as it is.

Although the auto size of text input controls such as HTML’s `<input type=text>` and `<textarea>` elements is typically a fixed size, the contents of such elements can be used to determine a content-based intrinsic size, as for non-replaced block containers. The min-content and max-content keywords of the sizing properties thus represent content-based sizes for form controls which render their value as text contained within their box, allowing such controls to size to fit their visible contents similarly to regular non-replaced elements.

The content in this case is defined to be the input control’s values (the raw value in the case of `textarea`, or the value in the case of `input`), possibly transformed to a more human-readable and/or localized display format, which is then treated as child text runs of the input control, allowing soft wrap opportunities only where the input control would actually allow wrapping (whether keyed off of CSS properties or other, UA-internal constraints). If the input control has designated placeholder text to be overlaid in its value display area, then that text is also measured for the purpose of calculating the content-based size—whether or not the placeholder text is visible at the moment. (Thus the content-based intrinsic size of the input control is the larger of the size to fit the placeholder text and the size to fit the value.)

The UA may enforce a minimum (such as the size required to contain a single zero-width character, or the smallest usable size of a touch target) on the form control’s min-content and max-content sizes to ensure sufficient space for the caret and otherwise maintain usability of the form control.

Note: This might be extended to `iframe` or other content-containing replaced elements (see discussion), but text inputs are a major use-case; and being document-internal, have the least additional complications.

### 4.2. Intrinsic Contributions

A box’s min-content contribution/max-content contribution in each axis is the size of the content box of a hypothetical auto-sized float that contains only that box, if that hypothetical float’s containing block is zero-sized/infinitely-sized.

Note: This specification does not define precisely how to determine these sizes. Please refer to [CSS2], the relevant CSS specification for that display type, the rules for handling percentages (below), and/or existing implementations for further details.

## 5. Extrinsic Size Determination

Extrinsic sizing determines sizes based on the context of an element, without regard for its contents.

### 5.1. Percentage Sizing

Percentages specify sizing of a box with respect to the box’s containing block.

For example, in the following markup:

    <article style="height: 60em">
      <figure style="height: 50%;">
        <img style="height: 50%;">
      </figure>
    </article>

- the `<figure>` would be 30em tall = 50% of the definite 60em height of the `<article>`

- the `<img>` would be 15em tall = 50% of the `<figure>`’s height (which is itself definite because it’s a percentage resolved against a definite length)

Sometimes the size of a percentage-sized box’s containing block depends on the intrinsic size contribution of the box itself, creating a cyclic dependency. When calculating the intrinsic size contribution of such a box (including any calculations for a content-based automatic minimum size), a cyclic percentage—that is, a percentage value that would resolve against a containing block size which itself depends on that percentage—is resolved specially:

1. If the box is non-replaced, then the entire value of any max size property or preferred size property (width/max-width/height/max-height) specified as an expression containing a percentage (such as 10% or calc(10px + 0%)) that is cyclic is treated for the purpose of calculating the box’s intrinsic size contributions only as that property’s initial value. For example, given a box with width: calc(20px + 50%), its max-content contribution is calculated as if its width were auto. (The percentage is honored as usual, however, during the actual sizing of the box itself; see below.)
2. Likewise, if the box is replaced, then the entire value of any max size property or preferred size property specified as an expression containing a percentage that is cyclic is treated for the purpose of calculating the box’s max-content contributions only as that property’s initial value.
3. If the box is replaced, a cyclic percentage in the value of any max size property or preferred size property (width/max-width/height/max-height), is resolved against zero when calculating the min-content contribution in the corresponding axis. (See Compressible Replaced Elements for a list of which elements in HTML this applies to.) The UA may additionally floor this size based on UI considerations, such as ensuring certain UI elements remain visible (for example, the dropdown arrow on a `select`). This rule also applies when calculating a content-based automatic minimum size or its corresponding size contribution, yielding a definite “specified size suggestion”.
   For example, an `input` assigned width: calc(50% + 50px) has a min-content contribution of 50px, plus any horizontal margin/border/padding.

   Note: We are not 100% sure if zeroing out a percentage max-width on form controls is web-compatible. See Issue 765.

4. For the min size properties, as well as for margins and paddings, a cyclic percentage is resolved against zero for determining intrinsic size contributions.

Summary of the Cyclic-Percentage Intrinsic Size Contribution Rules (Above)
Element Type | Replaced | Non-replaced
--- | --- | ---
Contribution Type | min-content | max-content | min-content | max-content
min size & margin/padding | zeroᵈ | zeroᵈ | zeroᵈ | zeroᵈ
max & preferred size | zeroᶜ | initialᵇ | initialᵃ | initialᵃ

Then, unless otherwise specified, when calculating the used sizes and positions of the containing block’s contents:

- If the cyclic dependency was introduced due to a block-size or max-block-size on the containing block that causes it to depend on the size of its contents, the box’s percentage is not resolved and instead behaves as auto.

  Note: Grid items and flex items do allow percentages to resolve in this case.

- Otherwise, the percentage is resolved against the containing block’s size. (The containing block’s size is not re-resolved based on the resulting size of the box; the contents might thus overflow or underflow the containing block).

Note: These rules specify the previously-undefined behavior of this cyclic case in CSS2§10.2, CSS2§8.3, and CSS2§8.4. Note also, the behavior in CSS2§10.5 is superseded in their respective specifications for layout modes (such as flex layout) not described in CSS2.

For example, in the following markup:

    <article style="width: min-content">
      <aside style="width: 50%;">
      LOOOOOOOOOOOOOOOOOOOONG
      </aside>
    </article>

When calculating the width of the outer `<article>`, the inner `<aside>` behaves as width: auto, so the `<article>` sets itself to the width of the long word. Since the `<article>`’s width didn’t depend on "real" layout, though, it’s treated as definite for resolving the `<aside>`, whose width resolves to half that of the `<article>`.

In this example,

    <article style="height:auto">
      <aside style="height: 50%;">
        <div class=block style="height: 150px;"></div>
      </aside>
      <section style="height: 30px;"></section>
    </article>

because the percentage block size (height, in this case) on block-level elements is defined to not resolve inside content-sized containing blocks, the percentage height on the `<aside>` is ignored, that is, it behaves exactly as if auto were specified.

Letting percentages still resolve against a definite height when the min-height is intrinsic is an open issue. (CSS2 has a general statement about "height depending on contents", which this technically is, even though CSS2 didn’t have content-dependent keywords for min-height. Since this is new, we think we could have this different behavior.)

The following examples illustrate how block-axis percentages resolve against a containing block whose size depends on its contents.

    <article style="height:100px; min-height: min-content;">
      <aside style="height: 50%;">
        <div style="height: 150px;"></div>
      </aside>
      <section style="height: 30px;"></section>
    </article>

The initial height of the `<article>` is 100px, as specified, which would make the `<aside>` 50px tall when it resolved its percentage. However, we must calculate the min-height, by substituting it in for height. This causes the percentage on the `<aside>` to behave as auto, so the `<aside>` ends up 150px tall. The total height of the contents is thus 180px. This is larger than the specified 100px height, so the `<article>` gets adjusted to 180px tall.

Then, since the percentage could originally resolve against the (100px) height, it now resolves against the 180px height, so the `<aside>` ends up being 90px tall.

    <article style="height:auto; min-height: min-content;">
      <aside style="height: 50%;">
        <div class=block style="height: 150px;"></div>
      </aside>
      <section style="height: 30px;"></section>
    </article>

In this case, the percentage on the `<aside>` won’t normally resolve, because the containing block’s height is auto (and thus depends on the size of its contents). Instead it behaves as auto, resulting in a height of 150px for the `<aside>`, and an initial height of 180px for the `<article>` The min-height doesn’t change this; height: min-content; acts similarly to height: auto; and results in the same sizes.

    <article style="height:100px; min-height: min-content;">
      <aside style="height: 200%;">
        <div style="height: 150px;"></div>
      </aside>
      <section style="height: 30px;"></section>
    </article>

This is a variation on the first code block, and follows a similar path; the `<aside>` initially wants to compute to 200px tall (200% of the 100px containing block height). When we calculate the effects of min-height, the percentage behaves as auto, causing it to become 150px tall, and the total min-content height of the containing block to be 180px tall. Since this is larger than 100px, the `<article>` gets clamped to 180px, the percentage resolves against this new height, and the `<aside>` ends up being 360px tall, overflowing the `<article>`

## Compressible Replaced Elements

The following elements can have their min-content size compressed when their width/height or max-width/max-height is expressed with a percentage size. See § 4.2 Intrinsic Contributions.

- All replaced elements that aren’t form controls. See HTML§14.4. [HTML]

- `input` with any `type` that is not "button-like"; this can vary depending on the UA.

  A type is "button-like" in a particular UA if it displays similar to a `button` element, where it can contains actual content that determines the layout of the element. In most UAs, the "button", "reset", "submit", and "color" types are button-like; the "file" type is also partially button-like in some UAs, when it’s displayed as a combination of a text input (shrinkable) and a button (button-like, and thus not shrinkable).

- `select`, `textarea`, `progress`, `meter`.

## Changes

Major changes since the 4 March 2018 Working Draft include:

- Imported the box-sizing definition from CSS UI Level 3.
- More rigorously specified handling of cyclic percentages. (#1132, #2384, #2297, #2674)
- Changed the *-content values applied to the bock axis to not compute to the property’s initial value, but to rather “behave as” the property’s initial value. (#2708)
- Fixed miscellaneous trivial errors.

Major changes since the 7 February 2017 Working Draft include:

- More accurate definition of min-content and max-content sizes for replaced elements.
- Compute new keywords to the initial value, not to a potentially non-existent auto, when applied to the block axis.
- Specify that percent sizes on replaced elements zero out their min-content contribution.
- Fix confusing/wrong definition of percentage sizes resolved against a dependent containing block. (This may require further work.)
- Deferred the stretch and fit-content keywords to Level 4 to allow for further consideration of their behavior in indefinite containing blocks.
- Pulled in full definitions for all of the sizing properties (rather than diffing them): width, height, min-width, min-height, max-width', max-height, and box-sizing.

## Acknowledgments

Special thanks go to L. David Baron, Aaron Gustafson, Daniel Holbert, and Mats Palmgren for their contributions to this module.

## Privacy and Security Considerations

This specification introduces no new privacy or security considerations.

## Conformance

### Document conventions

Conformance requirements are expressed with a combination of descriptive assertions and RFC 2119 terminology. The key words “MUST”, “MUST NOT”, “REQUIRED”, “SHALL”, “SHALL NOT”, “SHOULD”, “SHOULD NOT”, “RECOMMENDED”, “MAY”, and “OPTIONAL” in the normative parts of this document are to be interpreted as described in RFC 2119. However, for readability, these words do not appear in all uppercase letters in this specification.

All of the text of this specification is normative except sections explicitly marked as non-normative, examples, and notes. [RFC2119]

Examples in this specification are introduced with the words “for example” or are set apart from the normative text with `class="example"`, like this:

This is an example of an informative example.

Informative notes begin with the word “Note” and are set apart from the normative text with `class="note"`, like this:

Note, this is an informative note.

Advisements are normative sections styled to evoke special attention and are set apart from other normative text with `<strong class="advisement">`, like this: UAs MUST provide an accessible alternative.

### Conformance classes

Conformance to this specification is defined for three conformance classes:

style sheet
  A CSS style sheet.
renderer
  A UA that interprets the semantics of a style sheet and renders documents that use them.
authoring tool
  A UA that writes a style sheet.

A style sheet is conformant to this specification if all of its statements that use syntax defined in this module are valid according to the generic CSS grammar and the individual grammars of each feature defined in this module.

A renderer is conformant to this specification if, in addition to interpreting the style sheet as defined by the appropriate specifications, it supports all the features defined by this specification by parsing them correctly and rendering the document accordingly. However, the inability of a UA to correctly render a document due to limitations of the device does not make the UA non-conformant. (For example, a UA is not required to render color on a monochrome monitor.)

An authoring tool is conformant to this specification if it writes style sheets that are syntactically correct according to the generic CSS grammar and the individual grammars of each feature in this module, and meet all other conformance requirements of style sheets as described in this module.

### Requirements for Responsible Implementation of CSS

The following sections define several conformance requirements for implementing CSS responsibly, in a way that promotes interoperability in the present and future.

#### Partial Implementations

So that authors can exploit the forward-compatible parsing rules to assign fallback values, CSS renderers must treat as invalid (and ignore as appropriate) any at-rules, properties, property values, keywords, and other syntactic constructs for which they have no usable level of support. In particular, user agents must not selectively ignore unsupported property values and honor supported values in a single multi-value property declaration: if any value is considered invalid (as unsupported values must be), CSS requires that the entire declaration be ignored.

#### Implementations of Unstable and Proprietary Features

To avoid clashes with future stable CSS features, the CSSWG recommends following best practices for the implementation of unstable features and proprietary extensions to CSS.

#### Implementations of CR-level Features

Once a specification reaches the Candidate Recommendation stage, implementers should release an unprefixed implementation of any CR-level feature they can demonstrate to be correctly implemented according to spec, and should avoid exposing a prefixed variant of that feature.

To establish and maintain the interoperability of CSS across implementations, the CSS Working Group requests that non-experimental CSS renderers submit an implementation report (and, if necessary, the testcases used for that implementation report) to the W3C before releasing an unprefixed implementation of any CSS features. Testcases submitted to W3C are subject to review and correction by the CSS Working Group.

Further information on submitting testcases and implementation reports can be found from on the CSS Working Group’s website at http://www.w3.org/Style/CSS/Test/. Questions should be directed to the public-css-testsuite@w3.org mailing list.

Support: Android Browser4+Baidu Browser7.12+Blackberry Browser10+Chrome10+Chrome for Android74+Edge12+Firefox29+Firefox for Android66+IE8+IE Mobile10+KaiOS Browser2.5+Opera9.5+Opera MiniAllOpera Mobile10+QQ Browser1.2+Safari5.1+Samsung Internet4+UC Browser for Android11.8+iOS Safari5.0+

Source: caniuse.com as of 2019-06-04

## Index

### Terms defined by this specification

- auto, in §3.2
- automatic minimum size, in §3.2
- automatic size, in §3.2
- available, in §2
- available block space, in §2
- available inline space, in §2
- available space, in §2
- behave as auto, in §3.2.1
- behaves as auto, in §3.2.1
- behaving as auto, in §3.2.1
- border-box, in §3.3
- box-sizing, in §3.3
- content-box, in §3.3
- cyclic percentage, in §5.1
- definite, in §2
- definite size, in §2
- Extrinsic sizing, in §5
- fallback, in §2
- fallback size, in §2
- fit-content block size, in §2.1
- fit-content inline size, in §2.1
- fit-content(<length-percentage>)
  - value for column-width, in §3.4
  - value for width, min-width, max-width, height, min-height, max-height, in §3.2
- fit-content size, in §2.1
- height
  - (property), in §3.1.1
  - definition of, in §3.1.1
- indefinite, in §2
- indefinite size, in §2
- inner size, in §2
- intrinsic size, in §2.1
- intrinsic size constraint, in §2.2
- intrinsic size contribution, in §2.2
- intrinsic sizes, in §4.1
- Intrinsic sizing, in §4
- <length-percentage>, in §3.2
- max-content
  - definition of, in §2.1
  - value for column-width, in §3.4
  - value for width, min-width, max-width, height, min-height, max-height, in §3.2
- max-content block size, in §2.1
- max-content block-size contribution, in §2.2
- max-content constraint, in §2.3
- max-content contribution, in §2.2
- max-content inline size, in §2.1
- max-content inline-size contribution, in §2.2
- max-content size, in §2.1
- max height, in §3.1.3
- max-height, in §3.1.3
- maximum height, in §3.1.3
- maximum size, in §3.1.3
- maximum width, in §3.1.3
- max size, in §3.1.3
- max size property, in §3.1.2
- max width, in §3.1.3
- max-width, in §3.1.3
- min-content
  - definition of, in §2.1
  - value for column-width, in §3.4
  - value for width, min-width, max-width, height, min-height, max-height, in §3.2
- min-content block size, in §2.1
- min-content block-size contribution, in §2.2
- min-content constraint, in §2.3
- min-content contribution, in §2.2
- min-content inline size, in §2.1
- min-content inline-size contribution, in §2.2
- min-content size, in §2.1
- min height, in §3.1.2
- min-height, in §3.1.2
- minimum height, in §3.1.2
- minimum size, in §3.1.2
- minimum width, in §3.1.2
- min size, in §3.1.2
- min size property, in §3.1.1
- min width, in §3.1.2
- min-width, in §3.1.2
- none, in §3.2
- outer size, in §2
- preferred height, in §3.1.1
- preferred size, in §3.1.1
- preferred size property, in §3.1
- preferred width, in §3.1.1
- size, in §2
- sizing property, in §3.1
- stretch fit, in §2
- stretch-fit block size, in §2.1
- stretch-fit inline size, in §2.1
- stretch-fit size, in §2.1
- width
  - (property), in §3.1.1
  - definition of, in §3.1.1

https://drafts.csswg.org/css-backgrounds-3/#propdef-border Referenced in:

- 3.3. Box Edges for Sizing: the box-sizing property

https://drafts.csswg.org/css-box-3/#border-box Referenced in:

- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 3.3. Box Edges for Sizing: the box-sizing property (2)

https://drafts.csswg.org/css-box-3/#content-box Referenced in:

- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 3.3. Box Edges for Sizing: the box-sizing property (2) (3) (4)

https://drafts.csswg.org/css-box-3/#margin-properties Referenced in:

- 5.1. Percentage Sizing

https://drafts.csswg.org/css-box-3/#propdef-padding Referenced in:

- 3.3. Box Edges for Sizing: the box-sizing property

https://drafts.csswg.org/css-box-3/#padding-properties Referenced in:

- 5.1. Percentage Sizing

https://drafts.csswg.org/css-cascade-4/#computed-value Referenced in:

- 3.3. Box Edges for Sizing: the box-sizing property

https://drafts.csswg.org/css-cascade-4/#initial-value Referenced in:

- 5.1. Percentage Sizing (2)

https://drafts.csswg.org/css-display-3/#block-box Referenced in:

- 3.2.2. Containing or Excluding Floats

https://drafts.csswg.org/css-display-3/#block-container Referenced in:

- 2.1. Auto Box Sizes

https://drafts.csswg.org/css-display-3/#box Referenced in:

- 2. Terminology (2)
- 3.3. Box Edges for Sizing: the box-sizing property

https://drafts.csswg.org/css-display-3/#containing-block Referenced in:

- 2. Terminology (2)
- 2.2. Intrinsic Size Contributions (2)
- 2.3. Intrinsic Size Constraints (2)
- 3.1.1. Preferred Size Properties: the width and height properties
- 3.1.2. Minimum Size Properties: the min-width and min-height properties
- 3.1.3. Maximum Size Properties: the max-width and max-height properties
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values (2)
- 5.1. Percentage Sizing (2)

https://drafts.csswg.org/css-display-3/#propdef-display Referenced in:

- 3.2.2. Containing or Excluding Floats

https://drafts.csswg.org/css-display-3/#display-type Referenced in:

- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values

https://drafts.csswg.org/css-display-3/#formatting-context Referenced in:

- 3.2.2. Containing or Excluding Floats

https://drafts.csswg.org/css-display-3/#initial-containing-block Referenced in:

- 2. Terminology (2)

https://drafts.csswg.org/css-display-3/#inline Referenced in:

- 3.1.1. Preferred Size Properties: the width and height properties

https://drafts.csswg.org/css-display-3/#inline-box Referenced in:

- 2.1. Auto Box Sizes

https://drafts.csswg.org/css-display-3/#non-replaced Referenced in:

- 3.1.1. Preferred Size Properties: the width and height properties
- 5.1. Percentage Sizing (2)

https://drafts.csswg.org/css-display-3/#replaced-element Referenced in:

- 5.1. Percentage Sizing (2) (3)
- Compressible Replaced Elements

https://drafts.csswg.org/css-display-3/#replaced-element Referenced in:

- 5.1. Percentage Sizing (2) (3)
- Compressible Replaced Elements

https://drafts.csswg.org/css-display-3/#text-run Referenced in:

- 4.1. Intrinsic Sizes

https://drafts.csswg.org/css-flexbox-1/#flex-item Referenced in:

- 5.1. Percentage Sizing

https://drafts.csswg.org/css-flexbox-1/#propdef-flex-basis Referenced in:

- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 3.3. Box Edges for Sizing: the box-sizing property

https://drafts.csswg.org/css-grid-1/#grid-item Referenced in:

- 5.1. Percentage Sizing

https://drafts.csswg.org/css-logical-1/#propdef-block-size Referenced in:

- 5.1. Percentage Sizing

https://drafts.csswg.org/css-logical-1/#propdef-max-block-size Referenced in:

- 5.1. Percentage Sizing

https://drafts.csswg.org/css-page-3/#at-ruledef-page Referenced in:

- 3.1. Sizing Properties

https://drafts.csswg.org/css-page-3/#descdef-page-size Referenced in:

- 3.1. Sizing Properties

https://drafts.csswg.org/css-text-3/#soft-wrap-opportunity Referenced in:

- 4.1. Intrinsic Sizes

https://drafts.csswg.org/css-values-3/#length-value Referenced in:

- 2. Terminology
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values (2)
- 3.3. Box Edges for Sizing: the box-sizing property

https://drafts.csswg.org/css-values-3/#percentage-value Referenced in:

- 2. Terminology
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values (2)
- 3.3. Box Edges for Sizing: the box-sizing property

https://drafts.csswg.org/css-values-4/#typedef-length-percentage Referenced in:

- 3.1.1. Preferred Size Properties: the width and height properties (2) (3)
- 3.1.2. Minimum Size Properties: the min-width and min-height properties (2) (3)
- 3.1.3. Maximum Size Properties: the max-width and max-height properties (2) (3)
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values (2) (3) (4)
- 3.3. Box Edges for Sizing: the box-sizing property (2) (3) (4)
- 3.4. New Column Sizing Values: the min-content, max-content, and fit-content() values (2) (3) (4)

https://drafts.csswg.org/css-values-4/#css-wide-keywords Referenced in:

- 1.2. Values

https://drafts.csswg.org/css-values-4/#comb-one Referenced in:

- 3.1.1. Preferred Size Properties: the width and height properties (2) (3) (4)
- 3.1.2. Minimum Size Properties: the min-width and min-height properties (2) (3) (4)
- 3.1.3. Maximum Size Properties: the max-width and max-height properties (2) (3) (4)
- 3.3. Box Edges for Sizing: the box-sizing property
- 3.4. New Column Sizing Values: the min-content, max-content, and fit-content() values (2)

https://drafts.csswg.org/css-writing-modes-4/#block-axis Referenced in:

- 2.1. Auto Box Sizes

https://drafts.csswg.org/css-writing-modes-4/#block-size Referenced in:

- 2. Terminology (2)
- 2.1. Auto Box Sizes
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values (2)
- 5.1. Percentage Sizing

https://drafts.csswg.org/css-writing-modes-4/#flow-relative Referenced in:

- 3.1. Sizing Properties

https://drafts.csswg.org/css-writing-modes-4/#inline-axis Referenced in:

- 2.1. Auto Box Sizes (2)
- 4.1. Intrinsic Sizes

https://drafts.csswg.org/css-writing-modes-4/#inline-size Referenced in:

- 2. Terminology (2)
- 2.1. Auto Box Sizes (2) (3) (4)

https://drafts.csswg.org/css-writing-modes-4/#physical Referenced in:

- 3.1.1. Preferred Size Properties: the width and height properties

https://drafts.csswg.org/css-writing-modes-4/#vertical-writing-mode Referenced in:

- 4.1. Intrinsic Sizes

https://drafts.csswg.org/css-writing-modes-4/#writing-mode Referenced in:

- 2. Terminology

https://drafts.csswg.org/css-multicol-1/#propdef-column-width Referenced in:

- 1.1. Module interactions
- 3.4. New Column Sizing Values: the min-content, max-content, and fit-content() values (2)

https://drafts.csswg.org/cssom-1/#dom-window-getcomputedstyle Referenced in:

- 3.3. Box Edges for Sizing: the box-sizing property

https://drafts.csswg.org/cssom-1/#resolved-value Referenced in:

- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values

https://html.spec.whatwg.org/multipage/form-elements.html#the-button-element Referenced in:

- 3.3. Box Edges for Sizing: the box-sizing property
- Compressible Replaced Elements

https://html.spec.whatwg.org/multipage/iframe-embed-object.html#the-iframe-element Referenced in:

- 4.1. Intrinsic Sizes

https://html.spec.whatwg.org/multipage/input.html#the-input-element Referenced in:

- 4.1. Intrinsic Sizes
- 5.1. Percentage Sizing
- Compressible Replaced Elements

https://html.spec.whatwg.org/multipage/form-elements.html#the-meter-element Referenced in:

- Compressible Replaced Elements

https://html.spec.whatwg.org/multipage/form-elements.html#the-progress-element Referenced in:

- Compressible Replaced Elements

https://html.spec.whatwg.org/multipage/form-elements.html#concept-textarea-raw-value Referenced in:

- 4.1. Intrinsic Sizes

https://html.spec.whatwg.org/multipage/form-elements.html#the-select-element Referenced in:

- 5.1. Percentage Sizing
- Compressible Replaced Elements

https://html.spec.whatwg.org/multipage/form-elements.html#the-textarea-element Referenced in:

- 4.1. Intrinsic Sizes
- Compressible Replaced Elements

https://html.spec.whatwg.org/multipage/input.html#attr-input-type Referenced in:

- Compressible Replaced Elements

### Terms defined by reference

- [css-backgrounds-3] defines the following terms:
  - border
- [css-box-3] defines the following terms:
  - border box
  - content box
  - margin properties
  - padding
  - padding properties
- [css-cascade-4] defines the following terms:
  - computed value
  - initial value
- [css-display-3] defines the following terms:
  - block box
  - block container
  - box
  - containing block
  - display
  - display type
  - formatting context
  - initial containing block
  - inline
  - inline box
  - non-replaced
  - replaced
  - replaced element
  - text run
- [css-flexbox-1] defines the following terms:
  - flex item
  - flex-basis
- [css-grid-1] defines the following terms:
  - grid item
- [CSS-LOGICAL-1] defines the following terms:
  - block-size
  - max-block-size
- [CSS-PAGE-3] defines the following terms:
  - @page
  - size
- [css-text-3] defines the following terms:
  - soft wrap opportunity
- [CSS-VALUES-3] defines the following terms:
  - <length>
  - <percentage>
- [css-values-4] defines the following terms:
  - <length-percentage>
  - css-wide keywords
  - |
- [css-writing-modes-4] defines the following terms:
  - block axis
  - block size
  - flow-relative
  - inline axis
  - inline size
  - physical
  - vertical writing mode
  - writing mode
- [CSS3COL] defines the following terms:
  - column-width
- [cssom-1] defines the following terms:
  - getComputedStyle(elt)
  - resolved value
- [HTML] defines the following terms:
  - button
  - iframe
  - input
  - meter
  - progress
  - raw value
  - select
  - textarea
  - type

## References

### Normative References

[CSS-BOX-3]
  Elika Etemad. CSS Box Model Module Level 3. 18 December 2018. WD. URL: https://www.w3.org/TR/css-box-3/
[CSS-CASCADE-4]
  Elika Etemad; Tab Atkins Jr.. CSS Cascading and Inheritance Level 4. 28 August 2018. CR. URL: https://www.w3.org/TR/css-cascade-4/
[CSS-DISPLAY-3]
  Tab Atkins Jr.; Elika Etemad. CSS Display Module Level 3. 28 August 2018. CR. URL: https://www.w3.org/TR/css-display-3/
[CSS-FLEXBOX-1]
  Tab Atkins Jr.; et al. CSS Flexible Box Layout Module Level 1. 19 November 2018. CR. URL: https://www.w3.org/TR/css-flexbox-1/
[CSS-LOGICAL-1]
  Rossen Atanassov; Elika Etemad. CSS Logical Properties and Values Level 1. 27 August 2018. WD. URL: https://www.w3.org/TR/css-logical-1/
[CSS-PAGE-3]
  Elika Etemad; Simon Sapin. CSS Paged Media Module Level 3. 18 October 2018. WD. URL: https://www.w3.org/TR/css-page-3/
[CSS-TEXT-3]
  Elika Etemad; Koji Ishii; Florian Rivoal. CSS Text Module Level 3. 12 December 2018. WD. URL: https://www.w3.org/TR/css-text-3/
[CSS-VALUES-3]
  Tab Atkins Jr.; Elika Etemad. CSS Values and Units Module Level 3. 31 January 2019. CR. URL: https://www.w3.org/TR/css-values-3/
[CSS-VALUES-4]
  Tab Atkins Jr.; Elika Etemad. CSS Values and Units Module Level 4. 31 January 2019. WD. URL: https://www.w3.org/TR/css-values-4/
[CSS-WRITING-MODES-4]
  Elika Etemad; Koji Ishii. CSS Writing Modes Level 4. 24 May 2018. CR. URL: https://www.w3.org/TR/css-writing-modes-4/
[CSS2]
  Bert Bos; et al. Cascading Style Sheets Level 2 Revision 1 (CSS 2.1) Specification. 7 June 2011. REC. URL: https://www.w3.org/TR/CSS2/
[CSS3COL]
  Florian Rivoal; Rachel Andrew. CSS Multi-column Layout Module Level 1. 28 May 2018. WD. URL: https://www.w3.org/TR/css-multicol-1/
[CSSOM-1]
  Simon Pieters; Glenn Adams. CSS Object Model (CSSOM). 17 March 2016. WD. URL: https://www.w3.org/TR/cssom-1/
[HTML]
  Anne van Kesteren; et al. HTML Standard. Living Standard. URL: https://html.spec.whatwg.org/multipage/
[RFC2119]
  S. Bradner. Key words for use in RFCs to Indicate Requirement Levels. March 1997. Best Current Practice. URL: https://tools.ietf.org/html/rfc2119

### Informative References

[CSS-BACKGROUNDS-3]
  Bert Bos; Elika Etemad; Brad Kemper. CSS Backgrounds and Borders Module Level 3. 17 October 2017. CR. URL: https://www.w3.org/TR/css-backgrounds-3/
[CSS-GRID-1]
  Tab Atkins Jr.; Elika Etemad; Rossen Atanassov. CSS Grid Layout Module Level 1. 14 December 2017. CR. URL: https://www.w3.org/TR/css-grid-1/
[CSS-UI-3]
  Tantek Çelik; Florian Rivoal. CSS Basic User Interface Module Level 3 (CSS3 UI). 21 June 2018. REC. URL: https://www.w3.org/TR/css-ui-3/

## Property Index

Name | Value | Initial | Applies to | Inh. | %ages | Animatable | Animation type | Canonical order | Computed value
--- | --- | --- | --- | --- | --- | --- | --- | --- | ---
box-sizing | content-box | border-box | content-box | all elements that accept width or height | no | N/A |  | discrete | per grammar | specified keyword
height | auto | <length-percentage> | min-content | max-content | fit-content(<length-percentage>) | auto | all elements except non-replaced inlines | no | relative to width/height of containing block |  | by computed value type, recursing into fit-content() | per grammar | as specified, with <length-percentage> values computed
max-height | none | <length-percentage> | min-content | max-content | fit-content(<length-percentage>) | none | all elements that accept width or height | no | relative to width/height of containing block | by computed value, recursing into fit-content() |  | per grammar | as specified, with <length-percentage> values computed
max-width | none | <length-percentage> | min-content | max-content | fit-content(<length-percentage>) | none | all elements that accept width or height | no | relative to width/height of containing block | by computed value, recursing into fit-content() |  | per grammar | as specified, with <length-percentage> values computed
min-height | auto | <length-percentage> | min-content | max-content | fit-content(<length-percentage>) | auto | all elements that accept width or height | no | relative to width/height of containing block | by computed value, recursing into fit-content() |  | per grammar | as specified, with <length-percentage> values computed
min-width | auto | <length-percentage> | min-content | max-content | fit-content(<length-percentage>) | auto | all elements that accept width or height | no | relative to width/height of containing block | by computed value, recursing into fit-content() |  | per grammar | as specified, with <length-percentage> values computed
width | auto | <length-percentage> | min-content | max-content | fit-content(<length-percentage>) | auto | all elements except non-replaced inlines | no | relative to width/height of containing block |  | by computed value type, recursing into fit-content() | per grammar | as specified, with <length-percentage> values computed

## Issues Index

This spec needs illustrations! See issue. ↵
We would like to define shorthands for each pair of sizing properties (e.g. width and height) but there is a naming conflict with the @page size descriptor [CSS-PAGE-3], so this has been deferred to Level 4. Suggestions on how to resolve this problem are welcome, see discussion. ↵
Replace this section with references to the new term automatic size. ↵
Letting percentages still resolve against a definite height when the min-height is intrinsic is an open issue. (CSS2 has a general statement about "height depending on contents", which this technically is, even though CSS2 didn’t have content-dependent keywords for min-height. Since this is new, we think we could have this different behavior.) ↵
#size Referenced in:

- 2.1. Auto Box Sizes (2) (3) (4) (5) (6) (7)

#inner-size Referenced in:

- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 3.3. Box Edges for Sizing: the box-sizing property (2) (3) (4)

#outer-size Referenced in:

- 2.1. Auto Box Sizes
- 2.2. Intrinsic Size Contributions
- 3.3. Box Edges for Sizing: the box-sizing property

#definite Referenced in:

- 2. Terminology (2) (3) (4)
- 2.1. Auto Box Sizes (2)
- 4.1. Intrinsic Sizes (2) (3) (4)
- 5.1. Percentage Sizing (2) (3) (4) (5)

#indefinite Referenced in:

- 2. Terminology (2)
- 2.1. Auto Box Sizes (2)
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 3.2.1. “Behaving as auto”
- Changes

#available Referenced in:

- 2. Terminology (2)
- 2.1. Auto Box Sizes (2) (3) (4) (5)
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values (2)
- 4.1. Intrinsic Sizes (2)

#stretch-fit Referenced in:

- 2. Terminology
- 2.1. Auto Box Sizes
- 4.1. Intrinsic Sizes (2)

#fallback Referenced in:

- 2. Terminology

#stretch-fit-size Referenced in:

- 2.1. Auto Box Sizes
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values

#max-content Referenced in:

- 2.1. Auto Box Sizes (2) (3)
- 2.2. Intrinsic Size Contributions
- 3.4. New Column Sizing Values: the min-content, max-content, and fit-content() values
- 4.1. Intrinsic Sizes (2) (3) (4) (5)

#max-content-inline-size Referenced in:

- 3.4. New Column Sizing Values: the min-content, max-content, and fit-content() values

#max-content-block-size Referenced in:

- 2.1. Auto Box Sizes

#min-content Referenced in:

- 2.1. Auto Box Sizes (2)
- 2.2. Intrinsic Size Contributions
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values (2)
- 3.4. New Column Sizing Values: the min-content, max-content, and fit-content() values
- 4.1. Intrinsic Sizes (2) (3)
- Compressible Replaced Elements

#min-content-inline-size Referenced in:

- 3.4. New Column Sizing Values: the min-content, max-content, and fit-content() values

#fit-content-size Referenced in:

- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values

#max-content-contribution Referenced in:

- 2.2. Intrinsic Size Contributions
- 2.3. Intrinsic Size Constraints
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 4.2. Intrinsic Contributions
- 5.1. Percentage Sizing (2) (3)

#min-content-contribution Referenced in:

- 2.2. Intrinsic Size Contributions
- 2.3. Intrinsic Size Constraints
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 4.2. Intrinsic Contributions
- 5.1. Percentage Sizing (2) (3) (4)

#intrinsic-size-contribution Referenced in:

- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 5.1. Percentage Sizing

#max-content-constraint Referenced in:

- 2. Terminology

#min-content-constraint Referenced in:

- 2. Terminology

#sizing-property Referenced in:

- 3.3. Box Edges for Sizing: the box-sizing property (2) (3) (4)
- 4.1. Intrinsic Sizes

#preferred-size-properties Referenced in:

- 5.1. Percentage Sizing (2) (3) (4)

#propdef-width Referenced in:

- 1. Introduction
- 1.1. Module interactions
- 3.1. Sizing Properties (2)
- 3.1.1. Preferred Size Properties: the width and height properties (2)
- 3.1.2. Minimum Size Properties: the min-width and min-height properties
- 3.1.3. Maximum Size Properties: the max-width and max-height properties
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values (2)
- 3.2.1. “Behaving as auto” (2)
- 3.3. Box Edges for Sizing: the box-sizing property (2) (3) (4)
- 4.1. Intrinsic Sizes (2)
- 5.1. Percentage Sizing (2) (3) (4) (5) (6)
- Compressible Replaced Elements
- Changes

#propdef-height Referenced in:

- 1. Introduction
- 1.1. Module interactions
- 3.1. Sizing Properties (2)
- 3.1.1. Preferred Size Properties: the width and height properties (2)
- 3.1.2. Minimum Size Properties: the min-width and min-height properties
- 3.1.3. Maximum Size Properties: the max-width and max-height properties
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 3.2.1. “Behaving as auto” (2)
- 3.3. Box Edges for Sizing: the box-sizing property (2) (3)
- 4.1. Intrinsic Sizes (2)
- 5.1. Percentage Sizing (2) (3) (4) (5) (6) (7)
- Compressible Replaced Elements
- Changes

#width Referenced in:

- 2. Terminology
- 3.3. Box Edges for Sizing: the box-sizing property

#height Referenced in:

- 2. Terminology
- 3.3. Box Edges for Sizing: the box-sizing property

#min-size-properties Referenced in:

- 5.1. Percentage Sizing (2)

#propdef-min-width Referenced in:

- 1.1. Module interactions
- 3.1. Sizing Properties
- 3.1.2. Minimum Size Properties: the min-width and min-height properties (2)
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 4.1. Intrinsic Sizes (2)
- Changes

#propdef-min-height Referenced in:

- 1.1. Module interactions
- 3.1. Sizing Properties
- 3.1.2. Minimum Size Properties: the min-width and min-height properties (2)
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 4.1. Intrinsic Sizes (2)
- 5.1. Percentage Sizing (2) (3)
- Changes

#min-width Referenced in:

- 3.3. Box Edges for Sizing: the box-sizing property
- 4.1. Intrinsic Sizes (2)

#min-height Referenced in:

- 3.3. Box Edges for Sizing: the box-sizing property

#max-size-properties Referenced in:

- 5.1. Percentage Sizing (2) (3) (4)

#propdef-max-width Referenced in:

- 1.1. Module interactions
- 3.1. Sizing Properties
- 3.1.3. Maximum Size Properties: the max-width and max-height properties (2)
- 5.1. Percentage Sizing (2) (3)
- Compressible Replaced Elements

#propdef-max-height Referenced in:

- 1.1. Module interactions
- 3.1. Sizing Properties
- 3.1.3. Maximum Size Properties: the max-width and max-height properties (2)
- 5.1. Percentage Sizing (2)
- Compressible Replaced Elements
- Changes

#max-width Referenced in:

- 3.3. Box Edges for Sizing: the box-sizing property
- 4.1. Intrinsic Sizes (2)

#max-height Referenced in:

- 3.3. Box Edges for Sizing: the box-sizing property

#valdef-width-auto Referenced in:

- 2. Terminology
- 2.1. Auto Box Sizes
- 3.1.2. Minimum Size Properties: the min-width and min-height properties
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 3.2.1. “Behaving as auto” (2) (3) (4)
- 3.3. Box Edges for Sizing: the box-sizing property
- 4.1. Intrinsic Sizes (2) (3) (4)
- 4.2. Intrinsic Contributions
- 5.1. Percentage Sizing (2) (3)
- Changes

#automatic-size Referenced in:

- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values (2)
- 3.2.1. “Behaving as auto”

#automatic-minimum-size Referenced in:

- 5.1. Percentage Sizing (2)

#valdef-max-width-none Referenced in:

- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values

#valdef-width-min-content Referenced in:

- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values (2) (3) (4)
- 3.3. Box Edges for Sizing: the box-sizing property
- 4.1. Intrinsic Sizes
- 5.1. Percentage Sizing

#valdef-width-max-content Referenced in:

- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values (2) (3) (4)
- 4.1. Intrinsic Sizes

#valdef-width-fit-content-length-percentage Referenced in:

- 3.1.1. Preferred Size Properties: the width and height properties
- 3.1.2. Minimum Size Properties: the min-width and min-height properties
- 3.1.3. Maximum Size Properties: the max-width and max-height properties
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values (2) (3)
- 3.3. Box Edges for Sizing: the box-sizing property

#behave-as-auto Referenced in:

- 3.2.1. “Behaving as auto”
- 4.1. Intrinsic Sizes (2)
- 5.1. Percentage Sizing (2) (3) (4)

#propdef-box-sizing Referenced in:

- 1.1. Module interactions
- 3.2. Sizing Values: the <length-percentage>, auto | none, min-content, max-content, and fit-content() values
- 3.3. Box Edges for Sizing: the box-sizing property (2) (3) (4) (5)
- Changes (2)

#valdef-box-sizing-border-box Referenced in:

- 3.3. Box Edges for Sizing: the box-sizing property (2)

#valdef-column-width-min-content Referenced in:

- 3.4. New Column Sizing Values: the min-content, max-content, and fit-content() values

#valdef-column-width-max-content Referenced in:

- 3.4. New Column Sizing Values: the min-content, max-content, and fit-content() values

#valdef-column-width-fit-content-length-percentage Referenced in:

- 3.4. New Column Sizing Values: the min-content, max-content, and fit-content() values

#cyclic-percentage Referenced in:

- 5.1. Percentage Sizing (2) (3) (4)
- Changes